package ru.dbotthepony.mc.otm.capability;

import net.minecraft.nbt.CompoundTag;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.food.FoodData;
import net.minecraftforge.common.util.LazyOptional;
import net.minecraftforge.event.AttachCapabilitiesEvent;
import net.minecraftforge.event.entity.player.PlayerEvent;
import net.minecraftforge.eventbus.api.SubscribeEvent;
import net.minecraftforge.fmllegacy.network.PacketDistributor;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.Registry;
import ru.dbotthepony.mc.otm.network.AndroidCapabilityChangePacket;
import ru.dbotthepony.mc.otm.network.MatteryNetwork;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.math.BigDecimal;

public class AndroidCapabilityPlayer extends AndroidCapability {
	private boolean is_android = false;
	private boolean network_is_android = false;

	@Override
	public void invalidateNetworkState() {
		super.invalidateNetworkState();
		network_is_android = false;
	}

	@Override
	public void readChanges(AndroidCapabilityChangePacket packet) {
		super.readChanges(packet);

		if (packet.is_android != null) {
			is_android = packet.is_android;
		}
	}

	@Nullable
	@Override
	public AndroidCapabilityChangePacket writeChanges() {
		AndroidCapabilityChangePacket packet = super.writeChanges();

		if (is_android != network_is_android) {
			network_is_android = is_android;
			if (packet == null) packet = new AndroidCapabilityChangePacket();
			packet.isAndroid(is_android);
		}

		return packet;
	}

	@Override
	public boolean isAndroid() {
		return is_android;
	}

	public void becomeAndroid() {
		if (is_android)
			return;

		is_android = true;
		energy_stored = new BigDecimal(32_000);
		energy_stored_max = new BigDecimal(32_000);

		ply.setHealth(0.0F);
		ply.die(Registry.DAMAGE_BECOME_ANDROID);
	}

	@Override
	public void deserializeNBT(CompoundTag compound) {
		super.deserializeNBT(compound);

		if (compound.contains("is_android"))
			is_android = compound.getByte("is_android") > 0;
	}

	@Nonnull
	@Override
	public CompoundTag serializeNBT() {
		CompoundTag tag = super.serializeNBT();
		tag.putByte("is_android", is_android ? (byte) 1 : (byte) 0);
		return tag;
	}

	public void becomeHumane() {
		if (!is_android)
			return;

		is_android = false;

		energy_stored = new BigDecimal(0);
		energy_stored_max = new BigDecimal(32_000);

		dropBattery();

		ply.setHealth(0.0F);
		ply.die(Registry.DAMAGE_BECOME_HUMANE);
	}

	@SubscribeEvent
	public static void onAttachCapabilityEvent(AttachCapabilitiesEvent<Entity> event) {
		if (event.getObject() instanceof Player ply) {
			event.addCapability(Registry.Names.ANDROID_CAPABILITY, new AndroidCapabilityPlayer(ply));
		}
	}

	@SubscribeEvent
	public static void onPlayerChangeDimensionEvent(PlayerEvent.PlayerChangedDimensionEvent event) {
		event.getPlayer().getCapability(MatteryCapability.ANDROID).ifPresent(IAndroidCapability::invalidateNetworkState);
	}

	@SubscribeEvent
	public static void onPlayerCloneEvent(PlayerEvent.Clone event) {
		event.getPlayer().getCapability(MatteryCapability.ANDROID).ifPresent((cap) -> {
			LazyOptional<IAndroidCapability> resolver = event.getOriginal().getCapability(MatteryCapability.ANDROID);

			if (!resolver.isPresent() || resolver.resolve().isEmpty()) {
				event.getOriginal().reviveCaps();
				resolver = event.getOriginal().getCapability(MatteryCapability.ANDROID);
			}

			if (!resolver.isPresent() || resolver.resolve().isEmpty()) {
				event.getOriginal().invalidateCaps();
				return;
			}

			cap.deserializeNBT(resolver.resolve().get().serializeNBT());
			cap.invalidateNetworkState();
			event.getOriginal().invalidateCaps();
		});
	}

	private final Player ply;

	public AndroidCapabilityPlayer(Player ent) {
		super(ent);
		ply = ent;
	}

	@Override
	protected void tickNetwork() {
		AndroidCapabilityChangePacket packet = writeChanges();

		if (packet != null)
			MatteryNetwork.ANDROID_PLAYER.send(PacketDistributor.PLAYER.with(() -> (ServerPlayer) ent), packet);
	}

	@Override
	public void tickInner() {
		super.tickInner();

		// TODO: Maybe passive drain?
		// extractEnergyInner(BigDecimal.valueOf(new Random().nextDouble()), false);

		FoodData stats = ply.getFoodData();

		if (stats.needsFood()) {
			int food_points_missing = 20 - stats.getFoodLevel();
			boolean food_changed = false;
			float saturation_points_missing = 20F - stats.getSaturationLevel();
			boolean saturation_changed = false;

			while (food_points_missing > 0) {
				if (this.extractEnergyInner(ENERGY_FOR_HUNGER_POINT, true).compareTo(ENERGY_FOR_HUNGER_POINT) == 0) {
					this.extractEnergyInner(ENERGY_FOR_HUNGER_POINT, false);
					food_points_missing--;
					food_changed = true;
				} else {
					break;
				}
			}

			while (saturation_points_missing >= 1F) {
				if (this.extractEnergyInner(ENERGY_FOR_HUNGER_POINT, true).compareTo(ENERGY_FOR_HUNGER_POINT) == 0) {
					this.extractEnergyInner(ENERGY_FOR_HUNGER_POINT, false);
					saturation_points_missing--;
					saturation_changed = true;
				} else {
					break;
				}
			}

			if (food_changed) {
				stats.setFoodLevel(20 - food_points_missing);
			}

			if (saturation_changed) {
				stats.setSaturation(20F - saturation_points_missing);
			}
		}
	}
}
