package ru.dbotthepony.mc.otm.screen;

import com.mojang.blaze3d.systems.RenderSystem;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.client.gui.screens.inventory.AbstractContainerScreen;
import net.minecraft.client.renderer.GameRenderer;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.FormattedText;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.entity.player.Inventory;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.menu.FormattingHelper;
import ru.dbotthepony.mc.otm.menu.MatteryMenu;
import ru.dbotthepony.mc.otm.menu.PoweredMatteryMenu;

import java.math.BigDecimal;
import java.util.List;
import java.util.function.Supplier;

public class MatteryScreen<T extends MatteryMenu> extends AbstractContainerScreen<T> {
	protected static final ResourceLocation CONTAINER_BASE = new ResourceLocation(OverdriveThatMatters.MOD_ID, "textures/gui/mattery_machine_base.png");

	protected ResourceLocation CONTAINER_BACKGROUND() {
		return CONTAINER_BASE;
	}

	public MatteryScreen(T p_97741_, Inventory p_97742_, Component p_97743_) {
		super(p_97741_, p_97742_, p_97743_);

		imageWidth = 176;
		imageHeight = 178;
		titleLabelY = 4;

		inventoryLabelY = imageHeight - 92;
	}

	protected boolean hovering_power_gauge = false;

	protected List<FormattedText> getFormattedPowerLevel(Supplier<BigDecimal> s_energy, Supplier<BigDecimal> s_max_energy) {
		BigDecimal energy = s_energy.get();
		BigDecimal max_energy = s_max_energy.get();

		float level;

		if (max_energy.compareTo(BigDecimal.ZERO) == 0) {
			level = 0f;
		} else {
			level = energy.divide(max_energy, MatteryCapability.ROUND_RULES).floatValue();
		}

		return List.of(
			new TranslatableComponent("otm.gui.power.percentage_level", String.format("%.2f", level * 100d)),
			FormattingHelper.formatPowerLevel(energy, max_energy)
		);
	}

	protected boolean renderPowerGauge(PoseStack pose, int x, int y, int mouseX, int mouseY, Supplier<BigDecimal> s_energy, Supplier<BigDecimal> s_max_energy) {
		BigDecimal energy = s_energy.get();
		BigDecimal max_energy = s_max_energy.get();

		int i = (width - imageWidth) / 2;
		int j = (height - imageHeight) / 2;

		float level;

		if (max_energy.compareTo(BigDecimal.ZERO) == 0) {
			level = 0f;
		} else {
			level = energy.divide(max_energy, MatteryCapability.ROUND_RULES).floatValue();

			if (level >= 0.98)
				level = 1f;
		}

		if (level > 0.01) {
			RenderSystem.setShader(GameRenderer::getPositionTexShader);
			RenderSystem.setShaderColor(1.0F, 1.0F, 1.0F, 1.0F);
			RenderSystem.setShaderTexture(0, CONTAINER_BASE);

			int intHeight = (int) Math.floor(level * 46d + 0.5d);

			this.blit(pose, i + x, j + y + 46 - intHeight, 176, 0, 7, intHeight);
		}

		x += leftPos;
		y += topPos;
		hovering_power_gauge = x <= mouseX && x + 7 >= mouseX && y <= mouseY && y + 46 >= mouseY;
		return hovering_power_gauge;
	}

	@Override
	public void render(PoseStack p_98418_, int mouseX, int mouseY, float p_98421_) {
		this.renderBackground(p_98418_);
		super.render(p_98418_, mouseX, mouseY, p_98421_);
		this.renderTooltip(p_98418_, mouseX, mouseY);
	}

	@Override
	protected void renderBg(PoseStack pose, float p_97788_, int mouseX, int mouseY) {
		RenderSystem.setShader(GameRenderer::getPositionTexShader);
		RenderSystem.setShaderColor(1.0F, 1.0F, 1.0F, 1.0F);
		RenderSystem.setShaderTexture(0, CONTAINER_BACKGROUND());
		int i = (width - imageWidth) / 2;
		int j = (height - imageHeight) / 2;
		// PoseStack, x, y, image_start_x, image_start_y, rect_size_x, rect_size_y
		this.blit(pose, i, j, 0, 0, this.imageWidth, this.imageHeight);
	}
}
